#' toggles between ouch and ape format trees 
#' @param ot a phylogenetic tree in ouch or ape format
#' @param regimes if given in ape format, are appended to
#' phylo format as phy$regimes.  If the ouchtree is a fitted
#' hansen object, regimes will automatically be imported from it
#' unless other regime choice is given here.  
#' @param safe mode for going from ape to ouch.  After converting, this
#' writes to a temporary nexus file and reads the tree back in, because
#' phylo format does not have a unique specification for a unique tree,
#' resulting in all kinds of silly problems when developers haven't been careful. 
#' Defaults to true, and will clean up after itself.  
#' @return a phylogenetic tree in the opposite format
#' @export
convert <- function(ot, regimes=NULL, safe=TRUE){
  if(!is.null(regimes))
    safe<-FALSE # cannot write to nexus if regimes are required
	if(is(ot, "ouchtree")){

		n <- ot@nnodes
		# find the tips vs internals
		anc <- as.integer(ot@ancestors[!is.na(ot@ancestors)])
		internal <- sort(anc)[seq(2,n-1, by=2)]
		tmp <- integer(n)
		tmp[internal] = 1
		tips <- which(tmp == 0)

		root <- which(is.na(ot@ancestors))
		internal <- internal[internal!=root] #remove root form internal list
		
		new_ids <- integer(n)
		new_ids[tips] <- 1:length(tips)
		new_ids[root] <- length(tips)+1
		new_ids[internal] <- (length(tips)+2):n
		
		new_ancestor <- new_ids[as.integer(ot@ancestors)]

		edge <- matrix(NA, n-1, 2)
		edge[,1] <- new_ancestor[!is.na(new_ancestor)]
		edge[,2] <- new_ids[!is.na(new_ancestor)]

		anc <- as.integer(ot@ancestors[!is.na(ot@ancestors)])
		lengths <- ot@times[!is.na(ot@ancestors)] - ot@times[anc]
	
		labels <- ot@nodelabels[tips]

		tree <- list(edge=edge, Nnode = (n-1)/2, tip.label = labels, edge.length= lengths )
		class(tree) <- "phylo"

    if(safe){
      write.nexus(file="tmp3612411.nex", tree)
      tree <- read.nexus("tmp3612411.nex")
      unlink("tmp.nex")
    }

		if (is(ot, "hansentree")) {
			regimes <- ot@regimes[[1]][-1]
			tree$regimes <- regimes
		}

    if(!is.null(regimes)){
      tree$regimes <- regimes[-1]
    }

	} else	if (is(ot, "phylo")){ 
			tree <- ape2ouch(ot)
		}
#	plot(tree)
	tree
}




#' The anoles data set
#'
#' as from ouch with additional regimes added and minor formatting changes 
#' @name anoles
#' @docType data
# @references 
#' @keywords data
NULL

#' The phylogeny for the anoles data set
#'
#' The bimaculus phylogeny, as from the ouch package
#' @name tree
#' @docType data
# @references 
#' @keywords data
NULL



#' format data in ape format into ouch format
#' @param tree a phylogenetic tree of class "phylo", ape format, or a tree
#' in ouch format with the data also in ouch format
#' @param traits a numeric with trait values, or a matrix or data frame of 
#' traits, rownames matching species or handed in
#' @param species_names in the order of entries in traits, if not given in rownames.  
#' @param regimes the column in traits containing regime labels
#' @return the ouch-formatted tree, traits, and regimes  
#' @seealso \code{\link{convert}} to toggle between formats, including regime paintings
#' @details Should become an internal function to handle data conversion to ape-type
#' @import geiger
#' @export 
format_data <- function(tree, traits, species_names = NULL, regimes = NULL ){
# Function checks that tree and trait match and convert them into a format used by wrightscape
# Function also will code tree by finding the common ancestor of all species with matching entry specified in the regimes list and assigning that codename as the regime of all descendents of that ancestor.  May not handle conflicts if corresponding to overlapping clades.  Alternatively, the regimes can be specified directly in ouch format.   
	if( is(tree, "character" ) ){ 
		tree <- read.nexus(tree) 
	} else if (is(tree, "ouchtree")) {
		# uses my ouch2ape tree conversion script
    # should handle the data conversion of ouch2ape as well! 
		outtree <- convert(tree, regimes=regimes)
    if(is(traits, "data.frame"))
      dat <- traits[tree@nodelabels!="",1]
    else if(is(traits, "numeric"))
      dat <-  traits[tree@nodelabels!=""]
    else if(is(traits, "list"))
      dat <-  traits[[1]][tree@nodelabels!=""]
    else 
      stop(paste("traits class", class(traits), "not recognized"))
    names(dat) <- tree@nodelabels[!tree@nodelabels==""]
    out <- list(tree=outtree, data=dat)
	} else if(is(tree, "phylo")){
    # Figure out if species names is already attached to traits 
    if(is.null(species_names) ){
      if( !is.null(names(traits)) & is(traits, "numeric") ){
        species_names <- names(traits)
      } else if( !is.null(rownames(traits)) ){ 
        species_names <- rownames(traits) 
      } else { 
        stop("Species names not found")
      }
    }
    ## attach species names to traits
    if(is(traits, "numeric") ){
      names(traits) <- species_names
    } else if (is(traits, "data.frame") | is(traits, "matrix")){
    rownames(traits) <- species_names
    } else { stop("traits format unrecognized") }

    # drop missing taxa using geiger's function
    matched <- treedata(tree, traits, species_names)
    # treedata returns a matrix which loses class type for trait data.frame
    # so we restore the the dataframe structure
    if( is(matched$data, "matrix") ){ 
      matched$data <- as.data.frame(matched$data, stringsAsFactors=FALSE)
      if(is(traits, "data.frame")){
      for(i in 1:length(traits)){
          tmp <- class(traits[[i]])
          # each datarow that isn't numeric should be a character string (for regime, etc)
          if(tmp == "factor") tmp <- "character"
          class(matched$data[[i]]) <- tmp
        }
      }
    } 
    traits <- matched$data
    species_names <- rownames(matched$data) 
    # Convert to OUCH format 
    tree <- ape2ouch(matched$phy) 
    # Makes sure data is reformatted to ouch format matching the tree
    if( is(traits, "data.frame") ){
      message("traits ouch-formatted as data.frame")
      if(length(traits)>1){
          dataIn <- traits[match(tree@nodelabels, species_names),]
      } else {
        #hack to get around size-1 data-frames 
        traits[[2]] = NA
        dataIn <- traits[match(tree@nodelabels, species_names),]
        dataIn[[2]] <- NULL
      }
    } else if(is(traits, "numeric") ){ 
  message("traits ouch-formatted as numeric")
        dataIn <- traits[match(tree@nodelabels, species_names)]
    } else { stop(paste("data of class", class(traits), "not recognized")) }
    rownames(dataIn) <- tree@nodes
    if(is.numeric(regimes)){	
      R <- compute_regimes(tree, traits, species_names, regimes)
      nr <- R$noregimes
    } else {
      regimes= as.factor(rep(" ", length=tree@nnodes))
      names(regimes) <- tree@nodes 
      R <- list(regimes=regimes) 
      nr <- R$regimes
    }
	out <- list(tree=tree, data=dataIn, regimes=R$regimes, noregimes=nr)
	} else { 
    stop("Problem with tree format") 
  }
  out 
} 

#' internal method for computing the regimes used by format_data()
#' @param tree ouch-formated phylogenetic tree
#' @param traits ouch-formated data set
#' @param species_names for all the nodes 
#' @param regimes list in ouch format
#' @return a list of the regimes as edge-labels for ape's phylo format 
#' @keywords internal
compute_regimes <- function(tree, traits, species_names, regimes){

	## attach species names to regimes
	if(is.null(regimes) ){
		if( is(traits, "data.frame") | is(traits, "matrix") ){
			n <- dim(traits)[1]
		} else if (is(traits, "numeric")){
			n <- length(traits)
		}
		regimes <- as.factor(character(n))
		names(regimes) <- species_names
	} else if(length(regimes) == 1) {
		regimes <- traits[,regimes]
	}

	# order regimes into ouch format
	regimes <- regimes[match(tree@nodelabels, species_names)]

	# add an ancestral regime instead of NA, which confuses the mrcaOUCH function
	regimes <- as.character(regimes)
	regimes[is.na(regimes)] <- "anc"
	regimes <- as.factor(regimes)

	# add a noregimes case for ou1 fits
	noregimes <- as.factor(character(length(regimes)))

	# Paint regimes by clade using functions from maticce
	clades <- which(levels(regimes) != "anc")
	clade_ancestors <- integer(length(levels(regimes))-1)
	clade_names <- character(length(clade_ancestors) )
	k <- 1
	for(i in clades ){
		ancestor <-	as.integer(
			mrcaOUCH( tree@nodelabels[ regimes == levels(regimes)[i] ], tree)
		)
		clade_ancestors[k] <- ancestor
		clade_names[k] <- levels(regimes)[i]
		k <- k+1
	}
	## paintBranches needs the regime names in order
	A <- data.frame(clade_ancestors, clade_names)
	A <- A[order(A$clade_ancestors), ]
	regimes <- paintBranches(A$clade_ancestors, tree, regimeTitles=as.character(A$clade_names))

	# both regimes need node numbers as their names for OUCH
	names(noregimes) <- names(regimes)
	list(regimes=regimes, noregimes=noregimes)
}







# generics.R
# Author: Carl Boettiger <cboettig@gmail.com>
# License: BSD

#' S3 generic to provide a log-likelihood
#' @param x a model fit that can return a loglikelihood
#' @keywords internal
loglik <- function(x, ...) UseMethod("loglik")

#' S3 generic to extract the phylogeny  
#' @param x a model fit 
#' @keywords internal
get_phy <- function(x, ...) UseMethod("get_phy")

#' S3 generic to grab trait data
#' @param x a model fit 
#' @keywords internal
get_data <- function(x, ...) UseMethod("get_data")

#' S3 generic to provide the parameters 
#' @param x a model fit of interest 
#' @keywords internal
getParameters <- function(x, ...) UseMethod("getParameters")




#######################################################################
# All functions here are from "maticce" by Andrew Hipp, now obsolete  #
#######################################################################



# ---------------------------------------------------
# FUNCTIONS FOR PAINTING REGIMES ON AN S4 OUCH TREE #
# ---------------------------------------------------
# Adapted from functions used in Hipp 2007 Evolution paper
# Initially written for ouch v 1.2-4
# updated to ouch >= 2.4-2 Nov 2008
# updated to accommodate multiple trees Nov 2008

regimeVectors <-
# This is the basic call to get the full range of regimes over a set of trees
# Generates the list of painted branches representing all possible selective regimes for OU analyses, taking as argument
# species vectors that describe the clades at the bases of which regimes are specified to change.
# Arguments:
#  "tree" = the standard tree specification vectors of the OUCH-style tree
#  "cladeMembersList" = list of vectors containing names of the members of each clade (except for the root of the tree)
#  "maxNodes" = maximum number of nodes at which regime is permitted to change
# Value: 
#  "regList" = list of vectors that can each be plugged directly into OU analysis as the "regimes" argument
#  "nodeMatrix" = matrix of trees (rows) by nodes (columns) indicating what nodes are present on which trees
# 19 nov 08: changing to accept a list of trees and trimmed down greatly
function(ouchTrees, cladeMembersList, maxNodes = NULL) {
  nnode <- length(cladeMembersList)
  regMatrix <- regimeMatrix(n = nnode, maxNodes = maxNodes)
  apr = regimeMaker(ouchTrees, regMatrix, cladeMembersList)
  outdata <- list(regList = apr$regList, regMatrix = apr$regMatrix, nodeMatrix = apr$nodeMatrix)
  return(outdata) 
}


#' Paints branches with regimes changing at nodes specified
#'  
#' @param tree OUCH-style (S4) tree
#' @param regimeShiftNodes a vector of nodes or a list of taxa defining nodes at which selective regimes shift: 
#'         root may be included--if not present, will be added--but tips are meaningless in this context
#' @param regimeTitles a vector of titles for the regimes that begin at the root and at the nodes indicated in "regimeShiftNodes",
#'         in order of description in "regimeShiftNodes", except that the root is listed first in "regimeTitles"
#'         but not at all in "regimeShiftNodes"... defaults to "node[x]regime
#' @return a vector of regimes that can be handed to hansen
#' @export
paintBranches <-
# this is an especially ugly function, and I'm sure there are prettier ways of doing it. The 'paint' function in
# ouch could be adapted to this purpose, if one makes a series of consecutive calls going down the tree,
# and down the road it would probably make sense to turn this into just such a function.
function(regimeShiftNodes, tree, regimeTitles = NULL) {
  ## ------------------ begin ouchtree block -----------------
  ## check to see if tree inherits 'ouchtree'
  if (!is(tree,'ouchtree')) 
	stop(paste('This function has been rewritten to use the new S4 ', sQuote('ouchtree'), ' class.',
	'\nYou can generate a tree of this class by calling ', sQuote('ouchtree()'), '.', sep = ""))
  ## get the vectors we need:
  ancestor <- tree@ancestors # class = "character"
  node <- tree@nodes # class = "character"
  species <- tree@nodelabels # class = "character" -- note that nodelabels is more general than this indicates and the name should be changed throughout at some point
  times <- tree@times # class = "numeric"
  ## ------------------ end ouchtree block -------------------
  
  if(class(regimeShiftNodes) == "list") regimeShiftNodes <- unlist(lapply(regimeShiftNodes, mrcaOUCH, tree = tree))
  regimeShiftNodes <- unique(c(as.character(tree@root), regimeShiftNodes))
  if(identical(regimeTitles, NULL)) regimeTitles <- as.character(regimeShiftNodes)
  names(regimeTitles) = as.character(regimeShiftNodes)
  colorsVector = character(length(node))
  for (i in 1:length(ancestor)) {
    # First three lines fill up the vector for nodes that are hit in order
    if (is.na(ancestor[i])) {
      colorsVector[i] = regimeTitles["1"]
      next }
    if (as.character(ancestor[i]) %in% as.character(regimeShiftNodes)) {
      colorsVector[i] = regimeTitles[as.character(ancestor[i])]
      next }
    if (colorsVector[as.integer(ancestor[i])] != "") {
      colorsVector[i] = colorsVector[as.integer(ancestor[i])]
      next }
    # These lines fill up the vector for nodes run reached before their immediate ancestor
    nodeQ = integer(length(node))
    ii = i
    repeat {
      nodeQ = c(ii, nodeQ)
      ii = as.numeric(ancestor[ii])
      if (as.character(ancestor[ii]) %in% as.character(regimeShiftNodes)) {
        colorsVector[ii] = colorsVector[as.integer(ancestor[ii])]
        break}
      if (colorsVector[as.integer(ancestor[ii])] != "") {
        colorsVector[ii] = colorsVector[as.integer(ancestor[ii])]
        break} }
    
    for(j in nodeQ) {
      colorsVector[j] = colorsVector[as.integer(ancestor[j])] } 
      
      } # closes for(i in 1:length(ancestor)) loop 

      # a hack to fix a problem I don't understand... with the undesired side effect that it colors the stem of some subtrees rather than the crown as originally written
      for(i in 1:length(colorsVector)) if(colorsVector[i] == "") colorsVector[i] <- as.character(i) 
      
      # colors terminal branches if any terminal branches are in the regimeShiftNodes
      for(i in regimeShiftNodes) if(i %in% tree@term) colorsVector[as.numeric(i)] <- as.character(i)
      colorsVector <- as.factor(colorsVector)
      names(colorsVector) <- tree@nodes
  return(colorsVector) }

regimeMaker <- function(ouchTrees, regMatrix, nodeMembers) {
## supplants the old 'allPossibleRegimes'
## takes a list of ouchtree objects, a regimeMatrix ouput, and a list of nodeMembers (the taxa definining each node of interest)
## Value:
##  regList = a list of nodes defining the change points for each tree (i.e., a list of lists)
##  nodeMatrix = a matrix of trees (rows) by nodes (columns) indicating whether the node is present in each tree
  
  # set up variables
  numTrees <- length(ouchTrees)
  numNodes <- length(nodeMembers)
  if(numNodes != dim(regMatrix)[2]) stop('Number of nodes (columns) in regMatrix must equal number of items in nodeMembers list')
  nodeMatrix <- matrix(NA, nrow = numTrees, ncol = numNodes, dimnames = list(seq(numTrees), dimnames(regMatrix)[[2]]))
  changeNodes <- list(numTrees)
  regList <- list(numTrees)
  regMatrices <- list(numTrees)
  
  # fill outdata
  for(i in seq(numNodes)) nodeMatrix[, i] <- unlist(lapply(ouchTrees, isMonophyletic, taxa = nodeMembers[[i]]))
  for(i in seq(numTrees)) {
    tree <- ouchTrees[[i]]
    regMatrices[[i]] <- regMatrix * as.numeric(matrix(nodeMatrix[i, ], dim(regMatrix)[1], dim(regMatrix)[2], byrow = TRUE)) # multiplies regMatrix by nodes present
    regMatrices[[i]][1:(dim(regMatrices[[i]])[1] - 1), ][which(apply(regMatrices[[i]][1:(dim(regMatrices[[i]])[1] - 1), ], 1, sum) == 0), ] <- rep(NA, numNodes) # set to NA regimes that have no nodes, except for OU1 model
    regMatrices[[i]][duplicated(apply(regMatrices[[i]], 1, as.decimal)), ] <- rep(NA, numNodes) ## set to NA non-unique regimes
    dimnames(regMatrices[[i]]) <- list(seq(dim(regMatrices[[i]])[1]), dimnames(regMatrices[[i]])[[2]])
    numTreeRegs <- dim(regMatrices[[i]])[1]
    treeRegs <- list(numTreeRegs) # this will be assigned to regList[[i]]
    nodesVector <- unlist(lapply(nodeMembers, mrcaOUCH, tree = ouchTrees[[i]])) # as written, gets the MRCA for even invalid nodes just so indexing stays right
    for(j in seq(numTreeRegs)) {
      if(any(is.na(regMatrices[[i]][j, ]))) treeRegs[[j]] <- NA
      else {
        treeRegs[[j]] <- as.factor(paintBranches(c("1", nodesVector[as.logical(regMatrices[[i]][j, ])]), tree))
        names(treeRegs[[j]]) <- tree@nodes
      }
    }
    regList[[i]] <- treeRegs
  }
  regMatrices$overall <- regMatrix # this is the matrix that includes all regimes without regard to any tree
  outdata <- list(regList = regList, nodeMatrix = nodeMatrix, regMatrix = regMatrices)
  return(outdata)
}

regimeMatrix <- function(n, maxNodes) {
## recursive function that returns the same thing as oldRegimeMatrix, but much more efficient, at least for small maxNodes
## actually, it appears to be more efficient even at n = maxNodes
  if(n == 1) return(matrix(1:0, nrow = 2, ncol = 1))
  outmat <- matrix(NA, nrow = 0, ncol = n)
  for (i in 1:(n-1)) {
    temp <- c(rep(0, (i-1)), 1)
    remainder <- n - i
    if (maxNodes > 1 && remainder > 0) {
      nextMat <- regimeMatrix(remainder, maxNodes - 1)
      temp <- cbind(matrix(temp, dim(nextMat)[1], length(temp), byrow = TRUE), nextMat)
      }
    else temp[(i+1):n] <- rep(0, length((i+1):n))
    outmat <- rbind(outmat, temp)
  }
  outmat <- rbind(outmat, c(rep(0, n-1), 1))
  outmat <- rbind(outmat, rep(0,n))
  dimnames(outmat) = list(seq(dim(outmat)[1]), seq(dim(outmat)[2]))
  return(outmat)
}

as.decimal <- function(n) {
# takes a binary vector and makes it a decimal
  digits <- length(n)
  result <- 0
  for(i in digits:1) result <- result + n[i] * 2 ^ (digits - i)
  result
}




# Functions adapted from "maticce" by Andrew Hipp, now obsolete

# ------------------------------------------
# FUNCTIONS FOR TRAVERSING AN S4 OUCH TREE #
# ------------------------------------------

isMonophyletic <- function(tree, taxa) {
# returns T or F on whether a group of taxa is monophyletic in an ouch tree
  if(length(taxa) == 1) return(taxa %in% tree@nodelabels[tree@term])
  else(return(identical(sort(taxa), sort(nodeDescendents(tree, mrcaOUCH(taxa, tree))))))
}

nodeDescendents <- function(tree, startNode) {
## Recursive function to find all the descendents of a node on an 'ouchtree' object
  startNode <- as.character(startNode) # just to be safe
  daughterBranches <- as.character(tree@nodes[tree@ancestors %in% startNode])
  nodeNames <- tree@nodelabels[tree@nodes %in% daughterBranches]
  if(!identical(as.character(daughterBranches), character(0))) {
    for(i in daughterBranches) nodeNames <- c(nodeNames, nodeDescendents(tree, i))
  }
  return(nodeNames[nodeNames %in% tree@nodelabels[tree@term]])
}



#' Finds the most recent common ancestor on an OUCH formatted tree
#' @param cladeVector vector of species for which you want to find the most recent common ancestor
#' @param tree an ouchtree class phylogeny
#' @return the node number of the most recent common ancestor
#' @export
mrcaOUCH <-
# Finds most recent common ancestor for a vector of tips by:
#  1. Creating a vector of ancestral nodes leading to each tip
#  2. Creating an intersection set of ancestral nodes for all taxa by intersecting taxa successively with the last intersection set
#  3. Returning the node of the final intersection set that has the highest time
# Arguments:
#  "node" "ancestor" "times" "species" = the standard tree specification vectors of the OUCH-style tree
#  "cladeVector" = vector of species for which you want to find the most recent common ancestor
# Value: the node number of the most recent common ancestor
function(cladeVector, tree) {
  ## ------------------ begin ouchtree block -----------------
  ## check to see if tree inherits 'ouchtree'
  if (!is(tree,'ouchtree')) 
	stop(paste('This function has been rewritten to use the new S4 ', sQuote('ouchtree'), ' class.',
	'\nYou can generate a tree of this class by calling ', sQuote('ouchtree()'), '.', sep = ""))
  ## get the vectors we need:
  ancestor <- tree@ancestors # class = "character"
  node <- tree@nodes # class = "character"
  species <- tree@nodelabels # class = "character" -- note that nodelabels is more general than this indicates and the name should be changed throughout at some point
  times <- tree@times # class = "numeric"
  ## ------------------ end ouchtree block -------------------
  
  if(length(cladeVector) == 1) return(tree@nodes[tree@nodelabels == cladeVector])
  else {
    tips = match(cladeVector, species) 
    listOfAncestorLines = lapply(tips, ancestorLine, tree = tree) # 10 nov 08: this is identical to the appropriate subset of tree@lineages
    latestMatch = listOfAncestorLines[[1]]
    for (i in listOfAncestorLines) {
      latestMatch = i[match(latestMatch, i, nomatch = 0)] }
    timesVector = times[as.integer(latestMatch)]
    if(length(timesVector) == 1) {
      if (is.na(timesVector)) mrca = "1"
        else mrca = timesVector}
      else mrca = latestMatch[match(max(as.double(timesVector), na.rm = TRUE), timesVector)]
    return(mrca) 
  }
}

ancestorLine <-
# Creates a vector of ancestral nodes for a tip
# Arguments:
#  "tree" = an ouch-style (S4) tree
#  "tip" = the tip node to trace back
# Value: a vector of nodes leading from a tip to the root
# 10 nov 08: changed to just grab tree@lineages and make a vector that fits the old code
function(tip, tree) {
  ## check to see if tree inherits 'ouchtree'
  if (!is(tree,'ouchtree')) 
	stop(paste('This function has been rewritten to use the new S4 ', sQuote('ouchtree'), ' class.',
	'\nYou can generate a tree of this class by calling ', sQuote('ouchtree()'), '.', sep = ""))
  tip <- as.numeric(tip)
  nodesVector <- c(as.character(tree@lineages[[tip]][2:length(tree@lineages[[tip]])]), NA)
  return(nodesVector) 
  }


# pmc.R


#' Performs a phylogenetic monte carlo between modelA and modelB
#'
#' Simulates data under each model and returns the distribution of 
#' likelihood ratio, L(B)/L(A), under for both simulated datasets.
#' @param tree A phylogenetic tree.  Can be phylo (ape) or ouch tree
#' @param data The data matrix
#' @param modelA a model from the list, or a custom model, see details
#' @param modelB any other model from the list, or custom model, see details
#' @param optionsA a named list of arguments for fitting model A
#' @param optionsB a named list of arguments for fitting model B
#' @param nboot number of bootstrap replicates to use
#' @param ... additional arguments
#' @return 
#' list with the nboot likelihood ratios obtained from fitting both models
#' to data simulated by model A, and the nboot likelihood ratios obtained
#' by fitting both models to simulations from model B, and the likelihood 
#' ratio between the original MLE estimated models from the data. Also returns
#' each model object (A and B), and the estimates of each parameter for each
#' replicate in the element "par_dists", (see details), and call (the pmc 
#' function call used, for reference).  The returned object has S3 class pmc. 
#'
#' @details Possible models are all models from fitContinuous & ouch
#' 
#' The return value includes all parameters estimated under the structure
#' "par_dists", a data frame with columns "value", (the numerical value of
#' the parameter estimated) "parameter" (a factor indicating the name of the
#' parameter in the model, i.e. lambda), "comparison", (one of AA, AB, BA, or
#' BB, where the first letter indicates the class of model estimated, and 
#' the second indicates the model used to simulate the data on which the 
#' estimate is based. Hence AA is the bootstrap of model A, and BB the bootstrap
#' of model B.  Note that the cross-comparisons can also be informative).  
#' Finally "rep" indicates the replicate number for the simulation.  See examples
#' for plotting and calculating statistics from this data frame. 
#' @examples
#' require(geiger) # just for the sample data
#' data(geospiza)
#' attach(geospiza)
#' out <- pmc(geospiza.tree, geospiza.data[1], "BM", "lambda", nboot=5)
#' plot_pars(out$par_dists) # show the parameters  
#' ## Ex. mixing methods from packages -- data formats handled automatically 
#' ## Load Libraries ##
#' require(TreeSim) # to simulate a sample phyologeny
#' require(snowfall) # optional for parallelization
#' 
#' ## Simulate Data ##
#' simtree <- sim.bd.taxa(n=60, numbsim=1, lambda=1, mu=0, frac=1,
#'                        complete=FALSE, stochsampling=FALSE)[[1]][[1]] 
#' # simulates the EB model (by tranforming the tree)
#' dat <- rTraitCont(exponentialchangeTree(simtree, a=-0.5), sigma=5)
#' 
#' ## run PMC ## 
#' ## Specify the models and the options needed. 
#' regimes <- format_data(simtree, dat)$noregimes
#' eb_v_ou <- pmc(simtree, dat, modelA="EB", modelB="hansen", 
#'                optionsB=list(sqrt.alpha=1, sigma=1, regimes=regimes), 
#'                nboot=5) # small nboot just for example
#' plot(eb_v_ou) 
#' 
#' @import snowfall 
#' @import ouch
#' @export
# @enhance ouch
# @suggest TreeSim
pmc <- function(tree, data, 
                modelA = c("BM", "OU", "lambda", "kappa", "delta", "EB",
                           "white", "trend", "hansen", "brown"), 
                modelB = c("BM", "OU", "lambda", "kappa", "delta", "EB",
                           "white", "trend", "hansen", "brown"), 
                optionsA=list(), optionsB=list(), nboot=20, ...){

  A <- pmc_fit(tree, data, modelA, optionsA)
  B <- pmc_fit(tree, data, modelB, optionsB)
  lr_orig <- -2*(loglik(A) - loglik(B)) 

  reps <- sfLapply(1:nboot, function(i){

    null_sims <- simulate_and_update(A,B)
    AfitA <- null_sims[[1]]
    BfitA <- null_sims[[2]]

    ## Do the B sims
    test_sims <- simulate_and_update(B,A)
    BfitB <- test_sims[[1]] 
    AfitB <- test_sims[[2]]

    lrA <- -2*(loglik(AfitA) - loglik(BfitA)) 
    lrB <- -2*(loglik(AfitB) - loglik(BfitB))
    list(AA=as.list(c(lr=loglik(AfitA), getParameters(AfitA))), 
         BA=as.list(c(lr=loglik(BfitA), getParameters(BfitA))),
         AB=as.list(c(lr=loglik(AfitB), getParameters(AfitB))), 
         BB=as.list(c(lr=loglik(BfitB), getParameters(BfitB))))
  })
  ## some reformating, for convenience 
  dat <- melt(reps)
  names(dat) <- c("value", "parameter", "comparison", "rep")
  class(dat$value) <- "numeric"
  null = -2*(subset(dat, parameter=="lr" & comparison=="AA")$value -
             subset(dat, parameter=="lr" & comparison=="BA")$value)
  test = -2*(subset(dat, parameter=="lr" & comparison=="AB")$value - 
             subset(dat, parameter=="lr" & comparison=="BB")$value)
  output <- list(lr=lr_orig, null=null, test=test, par_dists=dat, 
                 A=A, B=B, call=match.call()) 
  class(output) <- "pmc"
  output
}


#' Internal helper function
#' Simulates under model A, updates both A and B based on that data
#' @param A model with simulate & update methods
#' @param B another model with simulate & update methods
#' @return a list of with fit of A on data simulated under A,
#' the fit on B on data simulated under A, and the simulated data itself
simulate_and_update <- function(A,B){
    error <- 1
    while(error){ ## Some sims don't converge. 

      simA <- simulate(A)
      simA <- matchformats(A, B, simA)
      AfitA <- try( update(A, data=simA[[1]]) )
      BfitA <- try( update(B, data=simA[[2]]) )

      if(!is(AfitA, "try-error") && !is(BfitA, "try-error") )
        error <- 0
      else
        warning("model fitting failed on data, simulating again")
    }
    list(AfitA, BfitA, simA)
}

#' Internal helper function to match data formats
#' @param A model with simulate & update methods
#' @param B another model with simulate & update methods
#' @param sim simulations produced by model A
#' @return a list of two data sets.  The first is in a format
#' appropriate to update model A, the second is in the format
#' approriate to update model B
matchformats <- function(A, B, sim){
  sim_for_updating_A <- sim
  sim_for_updating_B <- sim
  # reformat output from ouch's simulate method
  if (is(A, "ouchtree")){
    sim_for_updating_A <- sim$rep.1     
    sim_for_updating_B <- sim$rep.1
  }
  # The cross-comparison needs to check the data-formats match
  if(is(A, "ouchtree") & !is(B, "ouchtree")){
    ## this gets data into geiger format
    apeformatted <- format_data(get_phy(A), sim_for_updating_A)
    data <-apeformatted$data

    ## We just don't want data frame outputs to pass to geiger
    if(is(data, "data.frame")){
      tmp <- data[[1]] # we want numeric data
      names(tmp) <- rownames(data)
      data <- tmp
#    tmp <- na.exclude(data)
#    tmp2 <- as.numeric(tmp)
#    names(tmp2) <- names(tmp)
#    data <- tmp2
    }
    sim_for_updating_B <- data
  }
  # The cross-comparison needs to check the data-formats match
  if(!is(A, "ouchtree") & is(B, "ouchtree")){
    sim_for_updating_B <- format_data(get_phy(A),
                                      sim_for_updating_A)$data
  }
list(sim_for_updating_A, sim_for_updating_B)
}

#' plot the distributions
#' @param x a pmc object
#' @param ... Additional arguments: 
#'  A= a name for the first model in the pmc pairwise comparison
#'  B= a name for the second model in the pairwise comparison
#' @import ggplot2
#' @import reshape2
#' @method plot pmc
#' @S3method plot pmc
plot.pmc <- function(x, ...){
  df <- data.frame(x$null, x$test)
  colnames(df) <- c("null", "test")
  dat <- melt(df)
  # FIXME strange that ggplot wants 'value' and 'variable' to be unquoted!
  # Makes these appear to be undefined values & creates a warning in check  
  ggplot(dat) + geom_density(aes(value, fill=variable), alpha=.7) +
       geom_vline(x=x$lr, lwd=1, lty=2)
}

#' plot the parameter distributions
#' @param object a pmc object fit
#' @return a ggplot2 plot object
#' @export
plot_pars <- function(object){
## add lines for the estimated parameter values of A & B
  ggplot(object) + geom_boxplot(aes(comparison, value)) + facet_wrap(~parameter, scales="free_y") 
}



#' Fit any model used in PMC 
#'
#' The fitting function used by pmc to generalize fitting to any model
#' @param tree a phylogenetic tree. can be ouch or ape format
#' @param data trait data in ape or ouch format
#' @param model the name of the model to fit, see details for a list of 
#' currently supported types
#' @param options whatever additional options would be provided 
#' to the model fit, see details
#' @return a pmc_model object, anything that has methods "simulate", 
#' "update", getParameters, and getLikelihood
#' @details options should include all parameters required by the fit method
#' Currently methods avialable are fitContinuous (see geiger package) and 
#' hansen (see the ouch package).  
#' @examples 
#' ## a geiger example
#' require(geiger) # just to load the data
#' data(geospiza)
#' attach(geospiza)
#' lambdaFit<-pmc_fit(geospiza.tree, geospiza.data, model="lambda") 
#' ## Or a single trait at a time: 
#' lambdaFit<-pmc_fit(geospiza.tree, geospiza.data[1], model="lambda") 
#' ## an ouch example
#' require(ouch) # just for the data, 
#' data(bimac)
#' tree <- with(bimac,ouchtree(node,ancestor,time/max(time),species))
#' ou.3 <- pmc_fit(data=log(bimac['size']),tree, model="hansen", 
#'                 list(regimes=bimac['OU.3'],sqrt.alpha=1,sigma=1))
#' @import geiger 
#' @export
pmc_fit <- function(tree, data, model, options=list()){
  # Figure out if we need ape/geiger based formats or ouch formats
  fitContinuous_types <- c("BM", "OU", "lambda", "kappa", 
                           "delta", "EB", "white", "trend")
  if(model %in% fitContinuous_types){
    type <- "fitContinuous"  
  } else if(model %in% c("brown", "hansen")){
    type <- "hansen"
  } else {
    stop(paste("Model", model, "not recognized"))
  }
  ## Run a fitContinuous fit ##
  if(type == "fitContinuous"){
    # first, check data formats
    if(is(tree, "ouchtree")){
      # assumes data is same order as nodelabels
      if(is(data, "data.frame"))
        data <- data[[1]] # we want numeric data
      names(data) <- tree@nodelabels
      tmp <- na.exclude(data)
      tmp2 <- as.numeric(tmp)
      names(tmp2) <- names(tmp)
      data <- tmp2
      tree <- convert(tree)
    }
    args <- c(list(tree=tree, data=data, model=model), options) 
    object <- do.call(fitContinuous_object, args) 
  } else if(type == "hansen"){
    ## Fit an ouch object (hansen) 
    if(is(tree, "phylo")){
      tmp <- format_data(tree, data)
      tree <- tmp$tree
      data <- tmp$data
    }
    args <- c(list(data=data, tree=tree), options) 
   if(model == "hansen")
       object <- do.call(hansen, args)
   else if(model == "brown")
       object <- do.call(brown, args)
  } else {
    stop("Error: format not recognized.")
  }
  object
}

## powertest.R
# power curve
#' function to create the power curve for a tree 
#'
#' compares BM against OU of increasing value of alpha
#' @param tree an ouchtree class phylogeny
#' @param nboot number of bootstraps to use in power simulation
#' @param cpu number of cpu's available
#' @param threshold to use for the power calculation
#' @param alpha a sequence of alpha values to try
#' @param method currently ignored as only comparisons to OU through hansen are allowed
#' @return a power object, see examples in the vignette
#' @details function can take a while to run all comparisons.  
#' @export
treepower <- function(tree, nboot = 100, cpu = 2, threshold = .95, alpha = seq(0.001, 30, length=100), method="hansen"){
	## Gotta get templates for the models, do so by fitting some dummy data
	data <- c(rep(NA, tree@nnodes-tree@nterm), rnorm(tree@nterm))
	names(data) <- tree@nodes
	null <- brown(data,tree)
	regimes <- as.factor(rep("ns", tree@nnodes))
	names(regimes) <- tree@nodes
	test <- hansen(data,tree,regimes,1,1)

	## are we in parallel?
	if(cpu>1){ 	
		sfInit(parallel=TRUE, cpus=cpu) 
		sfLibrary(pmc)
		sfExportAll()
	} else sfInit()

	null_dist <- sfSapply(1:nboot, function(i){
		data <- simulate(null)$rep.1
		null <- update(null, data)
		test <- update(test, data)
		-2*(null@loglik - test@loglik) 
	})

	## Actually do the bootstraps for each alpha
	test_dist <- sfLapply(1:length(alpha), function(i){
		test@sqrt.alpha <- sqrt(alpha[i])
		sapply(1:nboot, function(i){
				data <- simulate(test)$rep.1
				null <- update(null, data)
				test <- update(test, data)
				-2*(null@loglik - test@loglik) 
		})
	})

	## Power calculation
	power <- sapply(1:length(alpha), function(i){
		threshold_tail <- sort(null_dist)[ round(threshold*nboot) ]
		sum(test_dist[[i]] > threshold_tail)/nboot
	})

	## format the output
    list(null_dist=null_dist, test_dist=test_dist, power=power, alpha=alpha, nboot=nboot, threshold=threshold, tree=tree)
}



#' flexible regime coloring for trees 
#' @param tree A phylo class or ouch-class tree
#' @param regimes in ouch format, (only needed if not already given in tree) 
#' @param colormap a standard colormap color, see the list
#' @param custom a custom colormap
#' @param rev logical, reverse the order of the colormap colors 
#' @return a set of colors than can be passed as edge.color in plot.phylo
#' @export
treepalette <- function(tree, regimes=NULL, colormap = c("rainbow", "heat.colors",
                        "terrain.colors", "topo.colors", "cm.colors", "gray"),
                        custom=NULL, rev=FALSE){

  if(!is.null(regimes) & is(tree,"ouchtree"))
    apetree <- convert(tree, regimes=regimes)
  else
    apetree <- tree

	colormap <- match.arg(colormap)
	if(colormap=="rainbow") 
    levels(apetree$regimes) <- rainbow(length(levels(apetree$regimes)))
	if(colormap=="heat.colors") 
    levels(apetree$regimes) <- heat.colors(length(levels(apetree$regimes)))
	if(colormap=="terrain.colors") 
    levels(apetree$regimes) <- terrain.colors(length(levels(apetree$regimes)))
	if(colormap=="topo.colors") 
    levels(apetree$regimes) <- topo.colors(length(levels(apetree$regimes)))
	if(colormap=="cm.colors") 
    levels(apetree$regimes) <- cm.colors(length(levels(apetree$regimes)))
	if(colormap=="gray"){
    n <-length(levels(apetree$regimes))+2
    levels(apetree$regimes) <- gray((1:n)/n)
  }
  if(!is.null(custom))
    levels(apetree$regimes) <- custom
	if(rev==TRUE) 
    levels(apetree$regimes) <- rev(levels(apetree$regimes))
	as.character(apetree$regimes)
} 



# file: object_oriented_geiger.R 
# author: Carl Boettiger <cboettig@gmail.com>
# license: BSD 
# Description: A set of functions to make Geiger's fits into objects that can have 
#   methods associated with them using R's S3 Classes



#  A geiger function they don't bother exposing. Copied here to access the 
# fitted value for the phylogenentic mean under BM.  
phylogMean<-function(phyvcv, data){
	o<-rep(1, length(data))
	ci<-solve(phyvcv)
	m1<-solve(t(o) %*% ci %*% o)
	m2<-t(o) %*% ci %*% data
	m1 %*% m2
}
	

#' wrapper for fitContinuous
#' 
#' Runs the fitContinuous function and returns and object with necessary
#' information to run methods such as 'update' and 'simulate'.  Call is the
#' same as the original fitContinuous function.
#' @param tree a phylo object phylogenetic tree
#' @param data ape data phylo format
#' @param model the type of model to fit
#' @param bounds optional list of bounds for the parameter fits
#' @param meserr measurment error, if any
#' @return an object of class fitContinuous 
## should take "fit" output option as post wrapper to a fit continuous,
## along with tree, data, and model.  If that fit is null, it can
## make the call to fitContinuous
fitContinuous_object <- function(tree, data, model="BM", bounds=NULL, 
                                 meserr=NULL){
  tmp <- treedata(tree,data)
  data <- tmp$data
  tree <- tmp$phy
  fit <- fitContinuous(phy=tree, data=data, model=model, 
                       bounds=bounds, meserr=meserr)
  for(i in 1:length(fit)){
    ## This makes lots of copies of the data if you have multiple 
    ## traits in a data frame.  For effiency, please don't run large
    ## numbers of traits in matrix form, but sapply over them instead.
    fit[[i]]$tree <- tree 
    fit[[i]]$data <- data
    fit[[i]]$model <- model
    fit[[i]]$bounds <- bounds
    fit[[i]]$meserr <- meserr
  	trans_tree <- transformTree(fit[[i]])

    if(model != "white"){
      if(is(data, "data.frame")){
        fit[[i]]$root <- phylogMean((fit[[i]]$beta) * 
                                     vcv.phylo(trans_tree), data[[i]])
      } else {
        fit[[i]]$root <- phylogMean((fit[[i]]$beta) * 
                                     vcv.phylo(trans_tree), data)
      }
    } else { ## "white" model
      fit[[i]]$root <- fit[[i]]$mean
      fit[[i]]$beta <- fit[[i]]$nv
    }
  }
	class(fit) <- "fitContinuous"
  fit
}

#' update a fitContinuous model
#' @param object the output of fitContinuous
#' @param ... data on which update will be based
#' @return the MLE estimated model for the given data
#' @method update fitContinuous
#' @S3method update fitContinuous
update.fitContinuous <- function(object, ...){
	fitContinuous_object(tree=object[[1]]$tree, ...,
                       model=object[[1]]$model,
                       bounds=object[[1]]$bounds, 
                       meserr=object[[1]]$meserr)
}


#' simulate method
#' @param object a fitContinuous object
#' @param nsim number of sims (currently always 1) 
#' @param seed an optional seed for the simulations (not implemented)
#' @param ... additional arguments, not implemented for fitContinuous simulations
#' @return simulated dataset
# These next lines would generate the the S3 method line for the NAMESPACE,
# but this causes a conflict with the S4 method so it has been removed.  
# instead we export this function
#' @method simulate fitContinuous
#' @S3method simulate fitContinuous
#' @details intended as an internal function, though an available S3 method
#' @import ape
# properly speaking we don't want to export this, but it solves the s4 conflict
#' @export
simulate.fitContinuous <- function(object, nsim=1, seed=NULL, ...){
# rTraitCont might not be ideal here, should warn if alpha is large!
#  Currently only designed to simulate from the first object
  data <- object[[1]]$data 
  for(i in 1:length(object)){
    if(object[[i]]$model != "OU"){
      tree <- transformTree(object[[i]]) 
    # transform the tree and apply the BM simulation method
      sim <- rTraitCont(tree, model="BM", sigma=sqrt(object[[i]]$beta),
                              root.value=object[[i]]$root)
      # Make sure the simulation species names are in the order given by the data
      # (rTraitCont gives them in the order given in the tree, which need not match!)
      # Thanks to Jason Weir for catching this.  
      data[, i] <- sim[match(rownames(data), names(sim))]
    } else if(object[[i]]$model == "white"){ 
      data[,i] <- rnorm(length(data[,i]), mean=mean(object[[i]]$data), sd = sqrt(object[[i]]$beta))
    } else { 
      # use untransfromed tree & with the OU simulation method
      data[,i] <- rTraitCont(object[[i]]$tree, model="OU", 
                             sigma=sqrt(object[[i]]$beta),
                             alpha=object[[i]]$alpha, theta=object[[i]]$root, 
                             root.value=object[[i]]$root)
    }
  }
  # Keep labels for traits and species on data 
  names(data) <- names(object[[1]]$data)
  rownames(data) <- rownames(object[[1]]$data)
	data
}

#' Method to extract the log likelihood
#' @param fit a fitContinuous object
#' @return the log likelihood, currently only for a single/first trait
#' @method loglik fitContinuous 
#' @S3method loglik fitContinuous  
#' @keywords internal
loglik.fitContinuous <- function(fit){
	fit[[1]]$lnl
}


#' Method to extract the parameters 
#' @param fit a fitContinuous object
#' @return a list of parameters, currently only for a single/first trait
#' @method getParameters fitContinuous  
#' @S3method getParameters fitContinuous  
#' @keywords internal
getParameters.fitContinuous <- function(fit){
	who <- match(c("lnl", "beta", "alpha", "root", "lambda", "k", "kappa", "delta", "a"),  names(fit[[1]]))
  pars <- fit[[1]][na.omit(who)]
  out <- as.numeric(pars)
  names(out) <- names(pars) 
  out
}


# Note that this is not multivariate, and is actually passed a fit[[i]] object
# This preserves the "vector" approach of geiger (which isn't actually 
# a vector/multivariate analysis so is stupid notation!) but is memory-inefficient 
# @keywords internal
transformTree <- function(fit){
	if(fit$model == "BM" | fit$model == "white") out <- fit$tree
	else if(fit$model == "OU") out <- ouTree(fit$tree, fit$alpha)
	else if(fit$model == "lambda") out <- lambdaTree(fit$tree, fit$lambda)
	else if(fit$model == "kappa") out <- kappaTree(fit$tree, fit$kappa)
	else if(fit$model == "delta") out <- deltaTree(fit$tree, fit$delta)
	else if(fit$model == "EB") out <- exponentialchangeTree(fit$tree, a=fit$a)
	else print(paste("Transform model ", fit$model, " not recognized"))
	out
}



#' Method to grab the phylogeny 
#' @return the phylogeny of the object 
#' @method get_phy fitContinuous 
#' @S3method get_phy fitContinuous
#' @keywords internal
get_phy.fitContinuous <- function(x, ...) x[[1]]$tree

#' Method to grab the data 
#' @return the trait data of the object 
#' @method get_data fitContinuous 
#' @S3method get_data fitContinuous
#' @keywords internal
get_data.fitContinuous <- function(x, ...) x[[1]]$data


## Add fitDiscrete functionality !! ###


## LASER doesn't actually provide simulation methods for most of its fit methods
# methods not written 

fitLaser <- function(object, method=c("pureBirth", "bd"), ...){
  branching.times(object)
}

update.fitLaser <- function(object, ...){ } 
simulate.fitLaser <- function(object, nsim=1, seed=NULL, ...){ }
getParameters.fitLaser<- function(fit){ }
loglik.fitLaser <- function(fit){}
get_data.fitLaser <- function(x, ...){}
get_phy.fitLaser <- function(x, ...){ }

# need a helper function to convert between a,r and b,d:
swp <- function(r, a){
    b = r/(1-a)
      d = r*a/(1-a)
        c(b,d)
}


#objects_ouch.R

## Ouch is really already object oriented, but needs a few additional methods.  
## Also needs to tweak the "update" method (since ouch simulate returns a list of replicates)


#' Method to extract the log likelihood 
#' @return the log likelihood  
#' @method loglik hansentree
#' @S3method loglik hansentree
#' @keywords internal
loglik.hansentree <- function(object) object@loglik

#' Method to extract the log likelihood 
#' @return the log likelihood  
#' @method loglik hansentree
#' @S3method loglik hansentree
#' @keywords internal
loglik.browntree <- function(object) object@loglik



#' Method to extract the parameters 
#' @return a list of parameters
#' @method getParameters hansentree
#' @S3method getParameters hansentree
#' @keywords internal
getParameters.hansentree <- function(object){
	c(sigma=object@sigma, unlist(object@theta), sqrt.alpha=object@sqrt.alpha)
}

#' Method to extract the parameters 
#' @return a list of parameters
#' @method getParameters browntree
#' @S3method getParameters browntree
#' @keywords internal
getParameters.browntree <- function(object){
	c(sigma=object@sigma, unlist(object@theta))
}


## ouch cannot simulate before it has fit to data.  
## this makes a tree that can be simulated directly from set parameters
## not strictly needed pmc
make_browntree <- function(tree, sigma, theta){
	class(tree) <- "browntree"
	tree@nchar <- as.integer(1)
	tree@sigma <- sigma
	tree@theta <- list(theta)
	tree@data <- list(numeric(tree@nnodes))
	tree
}

#' Method to grab the phylogeny 
#' @return the phylogeny of the object 
#' @method get_phy ouchtree 
#' @S3method get_phy ouchtree
#' @keywords internal
get_phy.ouchtree <- function(x, ...) x


#' Method to grab the data 
#' @return the trait data of the object 
#' @method get_data ouchtree 
#' @S3method get_data ouchtree
#' @keywords internal
get_data.ouchtree <- function(x, ...) x@data


## 
# methods not written 

fitTreePar <- function(object, ...){
}

update.fitTreePar <- function(object, ...){ } 
simulate.fitTreePar <- function(object, nsim=1, seed=NULL, ...){ }
getParameters.fitTreePar<- function(fit){ }
loglik.fitTreePar <- function(fit){}
get_data.fitTreePar <- function(x, ...){}
get_phy.fitTreePar <- function(x, ...){ }


